@@ -30,9 +30,13 @@ __all__ = [
 
 
 import cgi
-from email.message import Message
+try:
+    from email.message import Message
+except ImportError:
+    from email.Message import Message
 import simplejson
 from StringIO import StringIO
+import types
 import urllib
 from urlparse import urlparse, urljoin
 
@@ -196,8 +200,14 @@ class Resource(RestfulBase):
         self._ensure_representation()
         for suffix in ['_link', '_collection_link']:
             param = self._wadl_resource.get_parameter(
-                param_name + suffix, self.JSON_MEDIA_TYPE)
+                param_name + suffix)
             if param is not None:
+                try:
+                    value = param.get_value()
+                except KeyError, e:
+                    # The parameter could have been present, but isn't.
+                    # Try the next parameter.
+                    continue
                 if param.get_value() is None:
                     # This parameter is a link to another object, but
                     # there's no other object. Return None rather than
@@ -260,6 +270,7 @@ class Resource(RestfulBase):
         type_url = resource.type_url
         resource_type = urlparse(type_url)[-1]
         default = Entry
+
         if (type_url.endswith('-page')
             or (param_name is not None
                 and param_name.endswith('_collection_link'))):
@@ -330,15 +341,21 @@ class Resource(RestfulBase):
             # Get a representation of the linked resource.
             representation = self._root._browser.get(self._wadl_resource)
             representation = simplejson.loads(unicode(representation))
-            type_link = representation['resource_type_link']
-            if (type_link is not None
-                and type_link != self._wadl_resource.type_url):
-                # In rare cases, the resource type served by the
-                # server conflicts with the type the client thought
-                # this resource had. When this happens, the server
-                # value takes precedence.
-                resource_type = self._root._wadl.get_resource_type(type_link)
-                self._wadl_resource.tag = resource_type.tag
+
+            # In rare cases, the resource type served by the
+            # server conflicts with the type the client thought
+            # this resource had. When this happens, the server
+            # value takes precedence.
+            #
+            # XXX This should probably be moved into a hook method
+            # defined by Entry, since it's not relevant to other
+            # resource types.
+            if type(representation) == types.DictType:
+                type_link = representation['resource_type_link']
+                if (type_link is not None
+                    and type_link != self._wadl_resource.type_url):
+                    resource_type = self._root._wadl.get_resource_type(type_link)
+                    self._wadl_resource.tag = resource_type.tag
             self.__dict__['_wadl_resource'] = self._wadl_resource.bind(
                 representation, self.JSON_MEDIA_TYPE,
                 representation_needs_processing=False)
@@ -348,6 +365,16 @@ class Resource(RestfulBase):
         return not self == other
 
 
+class ScalarValue(Resource):
+    """A resource representing a single scalar value."""
+
+    @property
+    def value(self):
+        """Return the scalar value."""
+        self._ensure_representation()
+        return self._wadl_resource.representation
+    
+
 class HostedFile(Resource):
     """A resource representing a file managed by a lazr.restful service."""
 
@@ -385,9 +412,10 @@ class ServiceRoot(Resource):
     :ivar credentials: The credentials instance used to access Launchpad.
     """
 
-    # Custom subclasses of Entry or Collection to use when
+    # Custom subclasses of Resource to use when
     # instantiating resources of a certain WADL type.
-    RESOURCE_TYPE_CLASSES = {'HostedFile': HostedFile}
+    RESOURCE_TYPE_CLASSES = {'HostedFile': HostedFile,
+                             'ScalarValue': ScalarValue}
 
     def __init__(self, authorizer, service_root, cache=None,
                  timeout=None, proxy_info=None, version=None,
@@ -427,12 +455,12 @@ class ServiceRoot(Resource):
         """The value for the User-Agent header.
 
         This will be something like:
-        launchpadlib 1.6.1, lazr.restfulclient 1.0.0; oauth_consumer=apport
+        launchpadlib 1.6.1, lazr.restfulclient 1.0.0; application=apport
 
         That is, a string describing lazr.restfulclient and an
-        optional custom client built on top, and parameters
-        containing any authorization-specific information that
-        identifies the user agent (such as the OAuth consumer key).
+        optional custom client built on top, and parameters containing
+        any authorization-specific information that identifies the
+        user agent (such as the application name).
         """
         base_portion = "lazr.restfulclient %s" % __version__
         if self._base_client_name != '':
@@ -441,7 +469,9 @@ class ServiceRoot(Resource):
         message = Message()
         message['User-Agent'] = base_portion
         if self.credentials is not None:
-            for key, value in self.credentials.user_agent_params.items():
+            user_agent_params = self.credentials.user_agent_params
+            for key in sorted(user_agent_params):
+                value = user_agent_params[key]
                 message.set_param(key, value, 'User-Agent')
         return message['User-Agent']
 
@@ -711,9 +741,16 @@ class Collection(Resource):
         :return: length of the collection
         :rtype: int
         """
-        try:
-            return int(self.total_size)
-        except AttributeError:
+        total_size = self.total_size
+        if type(total_size) == types.IntType:
+            # The size was a number present in the collection
+            # representation.
+            return total_size
+        elif isinstance(total_size, ScalarValue):
+            # The size was linked to from the collection representation,
+            # not directly present.
+            return total_size.value
+        else:
             raise TypeError('collection size is not available')
 
     def __iter__(self):
